## -*- coding: utf-8 -*-
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

### This file is part of the Plone Weather software.
###
### (C) 2006 Emmanuel Coirier
###
### Plone Weather is a free software released under the GNU
### General Public License v2 as published by the Free Software Foundation.
###
### Please read and understand the LICENSE file placed in the doc directory.
###

import time
import re
import os


from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject

import Weather


class PloneWeatherForecastTool(PropertyManager, UniqueObject, SimpleItem) :
    """
        Enable Plone to ask a weather forecast server.
    """
    id = 'weather_forecast_tool'
    meta_type = 'Plone Weather Forecast Tool'
    plone_tool = 1
    title = "Plone Weather Forecast Tool"

    _properties = (
        {'id':'title', 'type': 'string', 'mode':'w'},
        {'id':'id', 'type': 'string', 'mode':'w'},
        {'id':'key', 'type': 'string', 'mode':'w'},
        {'id':'location', 'type': 'string', 'mode':'w'},
        {'id':'dayForecast', 'type': 'string', 'mode':'w'},

    )

    overviewPath = os.path.join(os.path.dirname(__file__), 'zmi', 'overview')

    manage_options = (

        ({'label' : 'Overview',
          'action' : 'overview'
          },)

        + PropertyManager.manage_options
        + SimpleItem.manage_options
    )

    security = ClassSecurityInfo()
    security.declareProtected(CMFCorePermissions.ManagePortal, 'overview')
    overview = PageTemplateFile(overviewPath)


    def manage_afterAdd(self, item, container) :
        """
            Create default structures
        """
        SimpleItem.manage_afterAdd(self, item, container)

        self.id = ""
        self.key = ""
        self.location = ""
        ## update every two hours (conforms to §8.2 of the weather.com's Weather XML Data Feed Implementation Guide)
        self.cacheDuration = 60 * 60 * 2
        self.dayForecast = 2

        self.cache = {
            "date" : 0,
            "dateForDetails" : 0,
        }

    def manageFormResults(self, **params) :
        """
            Update the conf values
            returns a tuple with the status message and the next state
        """

        self.id = params["id"]
        self.key = params["key"]

        purgeCache = False

        if self.location != params["location"] :
            ## weatherLocation has been changed by user
            try :
                Weather.checkLocation(params["location"], self.id, self.key)
                self.location = params["location"]
                purgeCache = True
            except RuntimeError, e :
                return ("Error : " + str(e), "error")

        if purgeCache and hasattr(self, "cache") :
            self.cache["date"] = 0
            self.cache["dateForDetails"] = 0

        return ("pwf_config_ok", "success")

    def searchLocation(self, location) :
        """
            Search for exact location given a town name.
            Return a tuple list with the code of the town and its exact name

            Note that since this method will be call for every call of the config
            template, passing None as the location will return None.
        """
        if location is None :
            return None
        return Weather.searchCity(location)

    def checkLocationFormat(self, location):
        return re.match("[a-zA-Z]{4}\d{4}", location)


    def getWeatherData(self) :
        """
            Return a dictionnary with the following keys :
            location : the location returned by the server, or an error message.
            iconUrl : an url of the icon to display (even if the message is an error)
            iconAlternativeText : a text describing the current weather
        """
        if self.cache["date"] + self.cacheDuration < time.time():
            ## cache has expired
            try:
                data = Weather.getSimpleWeather(
                    self.location,
                    self.id,
                    self.key
                )
                if data["forecast"] == "-" :
                    data["forecast"] = "na"
                self.cache["data"] = data
                self.cache["date"] = time.time()
                self.cache = self.cache
            except RuntimeError, e:
                return {
                    "location" : e,
                    "iconUrl" : "weather_icons/na.png",
                    "iconAlternativeText" : "Forecast is not available.",
                }

        else :
            data = self.cache["data"]

        return {
            "location" : data["location"],
            "iconUrl" : "weather_icons/%s.png" % data["forecast"],
            "iconAlternativeText" : data["forecastText"],
        }

    def getDetailedWeatherData(self) :
        """
            Return the complete dictionnary returned by Weather.py.
            More details can be found in the service DTD found at
            http://www.weather.com/documentation/xml/weather.dtd

            Run Weather.py if you want a sample dictionnary.
        """

        if self.cache["dateForDetails"] + self.cacheDuration < time.time():
            ## cache has expired
            try:
                data = Weather.getWeather(
                    self.location,
                    self.id,
                    self.key,
                    days=self.dayForecast,
                )
                self.cache["detailedData"] = data
                self.cache["dateForDetails"] = time.time()

            except RuntimeError, e:
                ## TODO : perhaps find a better return ?
                return None

        else :
            data = self.cache["detailedData"]

        return data


    def adminLink(self):
        return getSecurityManager().checkPermission('Manager', self)



InitializeClass(PloneWeatherForecastTool)
