#!/usr/bin/python
#################################################################################
#                                                                               #
#                       copyright (c) 2004 Ingeniweb SARL                       #
#                       copyright (c) 2006 Emmanuel Coirier                     #
#                                                                               #
#################################################################################

# LICENSE IS GPL

"""This library queries and parse weather forecast for a given city, using
weather.com webservices.

API Usage :

  # Ask the code for a given city (here we assume our city is the first checked on server
  myCityCode=searchCity("Marseille, France")[0][0]
  # ask the complete weather, using your partner_id and partner_key
  weather=getWeather(myCityCode, PARTNER_ID, PARTNER_KEY))
  # ask only some simple information
  weather=getSimpleWeather(myCityCode, PARTNER_ID, PARTNER_KEY)
  # Check wether a location is know or not
  try :
    if checkLocation(myCityCode, PARTNER_ID, PARTNER_KEY) :
      print "Cool, it's ok !"
  except RuntimeError, e :
    print "Location should not exist", e


IN ORDER THIS SERVICE TO WORK, YOU MUST GET YOUR (FREE) PARTNER_KEY and ID FROM :

http://registration.weather.com/registration/xmloap/step1

"""

from xml.dom import minidom
from urllib import quote, urlopen
import sys

SERVER_URL="http://xoap.weather.com"

########################################################
## UTILITY FUNCTIONS
########################################################

def getElementValue(node, element_name):
    """
    Given a dom node, returns string value of given element,
    (immediatly) found inside node.
    If element_name is not found, returns None.
    """
    elems=node.getElementsByTagName(element_name)
    if len(elems)>0:
        return elems[0].firstChild.data
    else:
        return None

def _get_error():
    type, value = sys.exc_info()[:2]
    return str(type), str(value)

########################################################
## WEATHER.COM API PRIMITIVE FUNCTIONS
########################################################

def askServer(query):
    """Ask the remote weather soap server. Returns the response as an
    XML DOM object"""
    # Build URL, get answer
    URL="%s/%s" % ( SERVER_URL, query)
    try:
        dataStream = urlopen(URL)
    except:
        error_message="Could not contact server ! [%s, %s]" % _get_error()
        raise RuntimeError, error_message
    # Parse XML answer, making a DOM tree
    try:
      doc=minidom.parse(dataStream).documentElement
    except:
        error_message="Could not parse response ! [%s, %s]" % _get_error()
        raise RuntimeError, error_message
    dataStream.close()
    # Try to catch errors
    errors=doc.getElementsByTagName('error')
    if doc.tagName=='error':
        error=doc.getElementsByTagName('err')[0]
        error_message="Server returned an error (code=%s) : '%s'" % (error.getAttribute("type"), error.firstChild.data)
        raise RuntimeError, error_message
    # return xml dom tree, finally
    return doc


########################################################
## WEATHER.COM XML DECODING
########################################################


def _decode_head(weather):
    head=weather.getElementsByTagName('head')[0]
    return {
      'locale' : getElementValue(head,'locale'),
      'text format' : getElementValue(head,'form'),
      'units' : {
        'temperature' : getElementValue(head,'ut'),
        'distance' : getElementValue(head,'ud'),
        'speed' : getElementValue(head,'us'),
          ## since weather.com server invert units, we reverts it here
        'precipitation' : getElementValue(head,'ur'),
        'pressure' : getElementValue(head,'up'),
      }
    }

def _decode_location(weather):
    loc=weather.getElementsByTagName('loc')[0]
    return {
      'id' : loc.getAttribute('id'),
      'name' : getElementValue(loc,'dnam'),
      'time_measure' : getElementValue(loc,'tm'),
      'latitude' : getElementValue(loc,'lat'),
      'longitude' : getElementValue(loc,'lon'),
      'sunrise' : getElementValue(loc,'sunr'),
      'sunset' : getElementValue(loc,'suns'),
      'timezone' : getElementValue(loc,'zone'),
    }

def _decode_wind(wind):
    return {
     'direction' : getElementValue(wind,'d'),
     'text_description' : getElementValue(wind,'d'),
     'max_wind_gust_speed' : getElementValue(wind,'gust'),
     'speed' : getElementValue(wind,'s'),
    }

def _decode_currentconditions(weather):
    try:
      cc=weather.getElementsByTagName('cc')[0]
    except:
      return None
    bar=cc.getElementsByTagName('bar')[0]
    wind=cc.getElementsByTagName('wind')[0]
    uv=cc.getElementsByTagName('uv')[0]
    return {
      'last_update' : getElementValue(cc,'lsup'),
      'observation_station' : getElementValue(cc,'obst'),
      'temperature' : getElementValue(cc,'tmp'),
      'feels_like_temperature' : getElementValue(cc,'flik'),
      'text_description' : getElementValue(cc,'t'),
      'icon_code' : getElementValue(cc,'icon'),
      'humidity' : getElementValue(cc,'hmid'),
      'visibility' : getElementValue(cc,'vis'),
      'dew_point' : getElementValue(cc,'dewp'),
      'pressure' : {
         'current_pressure' : getElementValue(bar,'r'),
         'text_description' : getElementValue(bar,'d'),
      },
      'wind' :  _decode_wind(wind),
      'uv' : {
         'indice' : getElementValue(uv,'i'),
         'text_description' : getElementValue(uv,'t'),
      }
    }

def _decode_day(day):
    parts=[]
    xparts=day.getElementsByTagName('part')
    for part in xparts:
      parts.append(_decode_daypart(part))
    return {
      'day' : day.getAttribute('d'),
      'month' : day.getAttribute('m'),
      'day_text' : day.getAttribute('t'),
      'date' : day.getAttribute('dt'),
      'sunset' : getElementValue(day, 'suns'),
      'sunrise' : getElementValue(day, 'sunr'),
      'average_temperature' : getElementValue(day, 'med'),
      'record_high_temperature' : getElementValue(day, 'rech'),
      'record_low_temperature' : getElementValue(day, 'recl'),
      'high_temperature' : getElementValue(day, 'hi'),
      'low_temperature' : getElementValue(day, 'low'),
      'average_precipitation' : getElementValue(day, 'pcp'),
      'parts': parts,
    }

def _decode_daypart(part):
    wind=part.getElementsByTagName('wind')[0]
    return {
      'day_part' : part.getAttribute('p'),
      'icon_code' : getElementValue(part,'icon'),
      'text_description' : getElementValue(part,'t'),
      'wind' : _decode_wind(wind),
      'percent_chance_precipitation' : getElementValue(part,'ppcp'),
      'humidity' : getElementValue(part,'hmid'),
    }


def _decode_dailyforecast(weather):
    dayf=weather.getElementsByTagName('dayf')[0]
    days=[]
    xdays=dayf.getElementsByTagName('day')
    for day in xdays:
      days.append(_decode_day(day))
    return {
      'last_update' : getElementValue(dayf,'lsup'),
      'days' : days,
    }

def _decode_link(link):
    return {
      'position': link.getAttribute('pos'),
      'text_description': getElementValue(link, 't'),
      'link': getElementValue(link, 'l'),
    }

def _decode_links(weather):
    links=[]
    xlinks=weather.getElementsByTagName('lnks')
    if len(xlinks)==0:
      return []
    else:
      xlinks=xlinks[0]
    for link in xlinks:
      links.append(_decode_link(link))
    return {
      'type' : xlinks.getAttribute('type'),
      'links' : links,
    }

########################################################
## WEATHER.COM HIGH-LEVEL API
########################################################

def searchCity(cityname):
    """Ask weather.com for the location id of the given cityname.
    Returns a list of possible ('locId', 'City Name')."""
    result=[]
    query="search/search?where=%s" % quote(cityname) ## escape undesirable characters
    answer=askServer(query)
    for loc in answer.getElementsByTagName('loc'):
        result.append((loc.getAttribute('id'), loc.firstChild.data))
    return result


def getWeather(locid, partner_id, partner_key, days=5, unit="m"):
    """ """
    query="weather/local/%s?prod=xoap&par=%s&key=%s&cc=*&unit=%s&dayf=%d&locale=fr_FR" % (
      quote(locid),
      quote(partner_id),
      quote(partner_key),
      unit,
      days
    )
    if partner_id == "" or partner_key == "" :
        raise RuntimeError("Id or key is void.")

    weather=askServer(query)
    head=_decode_head(weather)
    location=_decode_location(weather)
    cc=_decode_currentconditions(weather)
    dayf=_decode_dailyforecast(weather)
    links=_decode_links(weather)
    return {
        "head" : head,
        "location" : location,
        "cc" : cc,
        "dayf" : dayf,
        "links" : links
    }

def getSimpleWeather(locid, partner_id, partner_key, unit="m") :

    weather = getWeather(locid, partner_id, partner_key, 1, unit)
    return {
        "location" : weather["location"]["name"],
        "forecast" : weather["cc"]["icon_code"],
        "forecastText" : weather["cc"]["text_description"]
    }

def checkLocation(locid, partner_id, partner_key) :
    """
        Checks for good location by asking its weather.
        Raises an exception if something goes wrong
    """
    getWeather(locid, partner_id, partner_key, 1, "m")
    return True


########################################################
## TEST SUITE
########################################################

if __name__=="__main__":

    from pprint import pprint

    def printForecast(locid) :
        PARTNER_ID = ""         # put here your weather.com id (only for test purpose)
        PARTNER_KEY = ""        # put here your weather.com key (you can let these field empty if you don't plan to test this module independantly)

        if "-s" in sys.argv :
            pprint(getSimpleWeather(locid, PARTNER_ID, PARTNER_KEY))
        else :
            pprint(getWeather(locid, PARTNER_ID, PARTNER_KEY, days = 1))

    def test(cityName = "paris"):
        cities=searchCity(cityName)
        if len(cities) :
            pprint(cities)
            locid=cities[0][0]
            printForecast(locid)
        else :
            print "No city named %s at %s, trying code." % (cityName, SERVER_URL)
            printForecast(cityName)

    ok = False
    for cityName in sys.argv[1:] :
        if "-" in cityName :
            continue
        print "Using", cityName
        test(cityName)
        ok = True
    if not ok :
        print "No name given, using \"paris\"."
        test()
